@@ -105,8 +105,7 @@ STATIC mp_obj_t machine_timer_make_new(const mp_obj_type_t *type, size_t n_args,
         }
     }
 
-    machine_timer_obj_t *self = m_new_obj(machine_timer_obj_t);
-    self->base.type = &machine_timer_type;
+    machine_timer_obj_t *self = mp_obj_malloc(machine_timer_obj_t, &machine_timer_type);
     self->group = group;
     self->index = index;
 
@@ -134,13 +133,25 @@ STATIC void machine_timer_isr(void *self_in) {
 
     #if HAVE_TIMER_LL
 
-    #if CONFIG_IDF_TARGET_ESP32
+    #if CONFIG_IDF_TARGET_ESP32 && ESP_IDF_VERSION < ESP_IDF_VERSION_VAL(5, 0, 0)
     device->hw_timer[self->index].update = 1;
     #else
+    #if ESP_IDF_VERSION >= ESP_IDF_VERSION_VAL(4, 4, 0)
+    #if CONFIG_IDF_TARGET_ESP32S3
+    device->hw_timer[self->index].update.tn_update = 1;
+    #else
+    device->hw_timer[self->index].update.tx_update = 1;
+    #endif
+    #else
     device->hw_timer[self->index].update.update = 1;
     #endif
+    #endif
     timer_ll_clear_intr_status(device, self->index);
+    #if ESP_IDF_VERSION < ESP_IDF_VERSION_VAL(5, 0, 0)
     timer_ll_set_alarm_enable(device, self->index, self->repeat);
+    #else
+    timer_ll_set_alarm_value(device, self->index, self->repeat);
+    #endif
 
     #else
 
