@@ -26,33 +26,88 @@
 
 #include <mpconfigboard.h>
 
+#ifndef MICROPY_CONFIG_ROM_LEVEL
+
+// Set default feature levels for each processor
+
 #if defined(NRF51822)
-  #include "mpconfigdevice_nrf51822.h"
+#if defined(BLUETOOTH_SD)
+// If SoftDevice is used there is less flash/ram available for application
+#define MICROPY_CONFIG_ROM_LEVEL (MICROPY_CONFIG_ROM_LEVEL_MINIMUM)
+#else
+#define MICROPY_CONFIG_ROM_LEVEL (MICROPY_CONFIG_ROM_LEVEL_CORE_FEATURES)
+#endif
+
 #elif defined(NRF52832)
-  #include "mpconfigdevice_nrf52832.h"
+  #define MICROPY_CONFIG_ROM_LEVEL (MICROPY_CONFIG_ROM_LEVEL_BASIC_FEATURES)
+
 #elif defined(NRF52840)
-  #include "mpconfigdevice_nrf52840.h"
+#define MICROPY_CONFIG_ROM_LEVEL (MICROPY_CONFIG_ROM_LEVEL_EXTRA_FEATURES)
+
 #elif defined(NRF9160)
-  #include "mpconfigdevice_nrf9160.h"
+#define MICROPY_CONFIG_ROM_LEVEL (MICROPY_CONFIG_ROM_LEVEL_EXTRA_FEATURES)
+
 #else
   #pragma error "Device not defined"
 #endif
 
+#endif // MICROPY_CONFIG_ROM_LEVEL
+
+// pre-defined shortcuts to use below in #if queries or define values
+#define CORE_FEAT (MICROPY_CONFIG_ROM_LEVEL >= MICROPY_CONFIG_ROM_LEVEL_CORE_FEATURES)
+#define EXTRA_FEAT (MICROPY_CONFIG_ROM_LEVEL >= MICROPY_CONFIG_ROM_LEVEL_EXTRA_FEATURES)
+
 // options to control how MicroPython is built
+
 #ifndef MICROPY_VFS
-#define MICROPY_VFS                 (0)
+#define MICROPY_VFS                        (CORE_FEAT)
 #endif
+
+// micro:bit filesystem
+#ifndef MICROPY_MBFS
+#define MICROPY_MBFS                       (!MICROPY_VFS)
+#endif
+
+#ifndef MICROPY_ENABLE_SOURCE_LINE
+#define MICROPY_ENABLE_SOURCE_LINE         (CORE_FEAT)
+#endif
+
+#ifndef MICROPY_PY_ARRAY_SLICE_ASSIGN
+#define MICROPY_PY_ARRAY_SLICE_ASSIGN      (CORE_FEAT)
+#endif
+
+#ifndef MICROPY_PY_SYS_STDFILES
+#define MICROPY_PY_SYS_STDFILES            (CORE_FEAT)
+#endif
+
+#ifndef MICROPY_PY_UBINASCII
+#define MICROPY_PY_UBINASCII               (CORE_FEAT)
+#endif
+
+#ifndef MICROPY_PY_NRF
+#define MICROPY_PY_NRF                     (CORE_FEAT)
+#endif
+
+#ifndef MICROPY_HW_ENABLE_INTERNAL_FLASH_STORAGE
+#define MICROPY_HW_ENABLE_INTERNAL_FLASH_STORAGE (CORE_FEAT)
+#endif
+
+#ifndef MICROPY_EMIT_THUMB
+#define MICROPY_EMIT_THUMB          (EXTRA_FEAT)
+#endif
+
+#ifndef MICROPY_EMIT_INLINE_THUMB
+#define MICROPY_EMIT_INLINE_THUMB   (EXTRA_FEAT)
+#endif
+
 #define MICROPY_ALLOC_PATH_MAX      (512)
 #define MICROPY_PERSISTENT_CODE_LOAD (1)
-#define MICROPY_COMP_MODULE_CONST   (0)
-#define MICROPY_COMP_TRIPLE_TUPLE_ASSIGN (0)
 #define MICROPY_READER_VFS          (MICROPY_VFS)
 #define MICROPY_ENABLE_GC           (1)
 #define MICROPY_ENABLE_FINALISER    (1)
 #define MICROPY_STACK_CHECK         (1)
 #define MICROPY_HELPER_REPL         (1)
 #define MICROPY_REPL_INFO           (1)
-#define MICROPY_REPL_EMACS_KEYS     (0)
 #define MICROPY_REPL_AUTO_INDENT    (1)
 #define MICROPY_KBD_EXCEPTION       (1)
 #define MICROPY_LONGINT_IMPL        (MICROPY_LONGINT_IMPL_MPZ)
@@ -65,10 +120,6 @@
 #define MICROPY_ALLOC_GC_STACK_SIZE (32)
 #endif
 
-#define MICROPY_OPT_COMPUTED_GOTO   (0)
-#define MICROPY_OPT_CACHE_MAP_LOOKUP_IN_BYTECODE (0)
-#define MICROPY_OPT_MPZ_BITWISE     (0)
-
 // fatfs configuration used in ffconf.h
 #define MICROPY_FATFS_ENABLE_LFN       (1)
 #define MICROPY_FATFS_LFN_CODE_PAGE    437 /* 1=SFN/ANSI 437=LFN/U.S.(OEM) */
@@ -82,21 +133,25 @@
     #define MICROPY_FATFS_MAX_SS       (4096)
 #endif
 
-// TODO these should be generic, not bound to fatfs
+#if MICROPY_VFS
+// TODO these should be generic, not bound to a particular FS implementation
+#if MICROPY_VFS_FAT
+#define mp_type_fileio mp_type_vfs_fat_fileio
+#define mp_type_textio mp_type_vfs_fat_textio
+#elif MICROPY_VFS_LFS1
+#define mp_type_fileio mp_type_vfs_lfs1_fileio
+#define mp_type_textio mp_type_vfs_lfs1_textio
+#elif MICROPY_VFS_LFS2
+#define mp_type_fileio mp_type_vfs_lfs2_fileio
+#define mp_type_textio mp_type_vfs_lfs2_textio
+#endif
+#else // !MICROPY_VFS_FAT
 #define mp_type_fileio fatfs_type_fileio
 #define mp_type_textio fatfs_type_textio
-
-// use vfs's functions for import stat and builtin open
-#if MICROPY_VFS
-#define mp_import_stat mp_vfs_import_stat
-#define mp_builtin_open mp_vfs_open
-#define mp_builtin_open_obj mp_vfs_open_obj
 #endif
 
-// Enable micro:bit filesystem by default.
-#ifndef MICROPY_MBFS
-#define MICROPY_MBFS (1)
-#endif
+// Use port specific uos module rather than extmod variant.
+#define MICROPY_PY_UOS              (0)
 
 #define MICROPY_STREAMS_NON_BLOCK   (1)
 #define MICROPY_MODULE_WEAK_LINKS   (1)
@@ -104,41 +159,24 @@
 #define MICROPY_USE_INTERNAL_ERRNO  (1)
 #define MICROPY_PY_FUNCTION_ATTRS   (1)
 #define MICROPY_PY_BUILTINS_STR_UNICODE (1)
-#define MICROPY_PY_BUILTINS_STR_CENTER (0)
-#define MICROPY_PY_BUILTINS_STR_PARTITION (0)
-#define MICROPY_PY_BUILTINS_STR_SPLITLINES (0)
 #define MICROPY_PY_BUILTINS_MEMORYVIEW (1)
 #define MICROPY_PY_BUILTINS_FROZENSET (1)
-#define MICROPY_PY_BUILTINS_EXECFILE (0)
 #define MICROPY_PY_BUILTINS_COMPILE (1)
 #define MICROPY_PY_BUILTINS_HELP    (1)
 #define MICROPY_PY_BUILTINS_HELP_TEXT nrf5_help_text
 #define MICROPY_PY_BUILTINS_HELP_MODULES (1)
 #define MICROPY_MODULE_BUILTIN_INIT (1)
-#define MICROPY_PY_ALL_SPECIAL_METHODS (0)
 #define MICROPY_PY_MICROPYTHON_MEM_INFO (1)
-#define MICROPY_PY_BUILTINS_SLICE_ATTRS (0)
-#define MICROPY_PY_SYS_EXIT         (1)
 #define MICROPY_PY_SYS_MAXSIZE      (1)
-#define MICROPY_PY_SYS_STDIO_BUFFER (0)
-#define MICROPY_PY_COLLECTIONS_ORDEREDDICT (0)
-#define MICROPY_PY_MATH_SPECIAL_FUNCTIONS (0)
-#define MICROPY_PY_CMATH            (0)
-#define MICROPY_PY_IO               (0)
 #define MICROPY_PY_IO_FILEIO        (MICROPY_VFS_FAT || MICROPY_VFS_LFS1 || MICROPY_VFS_LFS2)
 #define MICROPY_PY_URANDOM          (1)
 #define MICROPY_PY_URANDOM_EXTRA_FUNCS (1)
-#define MICROPY_PY_UCTYPES          (0)
-#define MICROPY_PY_UZLIB            (0)
-#define MICROPY_PY_UJSON            (0)
-#define MICROPY_PY_URE              (0)
-#define MICROPY_PY_UHEAPQ           (0)
 #define MICROPY_PY_UTIME_MP_HAL     (1)
 #define MICROPY_PY_MACHINE          (1)
 #define MICROPY_PY_MACHINE_PULSE    (0)
+#define MICROPY_PY_MACHINE_SOFTI2C  (MICROPY_PY_MACHINE_I2C)
 #define MICROPY_PY_MACHINE_SPI      (0)
 #define MICROPY_PY_MACHINE_SPI_MIN_DELAY (0)
-#define MICROPY_PY_FRAMEBUF         (0)
 
 #ifndef MICROPY_HW_LED_COUNT
 #define MICROPY_HW_LED_COUNT        (0)
@@ -184,16 +222,51 @@
 #define MICROPY_PY_TIME_TICKS       (1)
 #endif
 
-#ifndef MICROPY_PY_NRF
-#define MICROPY_PY_NRF              (0)
-#endif
-
 #define MICROPY_ENABLE_EMERGENCY_EXCEPTION_BUF   (1)
 #define MICROPY_EMERGENCY_EXCEPTION_BUF_SIZE  (0)
 
-// if sdk is in use, import configuration
+// if sdk is in use, import configuration and enable some core features
 #if BLUETOOTH_SD
 #include "bluetooth_conf.h"
+#define MICROPY_BUILTIN_METHOD_CHECK_SELF_ARG (1)
+#define MICROPY_COMP_CONST                    (1)
+#define MICROPY_COMP_CONST_FOLDING            (1)
+#define MICROPY_COMP_CONST_LITERAL            (1)
+#define MICROPY_COMP_DOUBLE_TUPLE_ASSIGN      (1)
+#define MICROPY_CPYTHON_COMPAT                (1)
+#define MICROPY_ENABLE_COMPILER               (1)
+#define MICROPY_ENABLE_EXTERNAL_IMPORT        (1)
+#define MICROPY_ERROR_REPORTING               (2)
+#define MICROPY_FULL_CHECKS                   (1)
+#define MICROPY_GC_ALLOC_THRESHOLD            (1)
+#define MICROPY_MODULE_GETATTR                (1)
+#define MICROPY_MULTIPLE_INHERITANCE          (1)
+#define MICROPY_PY_ARRAY                      (1)
+#define MICROPY_PY_ASSIGN_EXPR                (1)
+#define MICROPY_PY_ASYNC_AWAIT                (1)
+#define MICROPY_PY_ATTRTUPLE                  (1)
+#define MICROPY_PY_BUILTINS_BYTEARRAY         (1)
+#define MICROPY_PY_BUILTINS_DICT_FROMKEYS     (1)
+#define MICROPY_PY_BUILTINS_ENUMERATE         (1)
+#define MICROPY_PY_BUILTINS_EVAL_EXEC         (1)
+#define MICROPY_PY_BUILTINS_FILTER            (1)
+#define MICROPY_PY_BUILTINS_MIN_MAX           (1)
+#define MICROPY_PY_BUILTINS_PROPERTY          (1)
+#define MICROPY_PY_BUILTINS_RANGE_ATTRS       (1)
+#define MICROPY_PY_BUILTINS_REVERSED          (1)
+#define MICROPY_PY_BUILTINS_SET               (1)
+#define MICROPY_PY_BUILTINS_SLICE             (1)
+#define MICROPY_PY_BUILTINS_STR_COUNT         (1)
+#define MICROPY_PY_BUILTINS_STR_OP_MODULO     (1)
+#define MICROPY_PY_COLLECTIONS                (1)
+#define MICROPY_PY_GC                         (1)
+#define MICROPY_PY_GENERATOR_PEND_THROW       (1)
+#define MICROPY_PY_MATH                       (1)
+#define MICROPY_PY_STRUCT                     (1)
+#define MICROPY_PY_SYS                        (1)
+#define MICROPY_PY_SYS_PATH_ARGV_DEFAULTS     (1)
+#define MICROPY_PY___FILE__                   (1)
+#define MICROPY_QSTR_BYTES_IN_HASH            (2)
 #endif
 
 #ifndef MICROPY_PY_UBLUEPY
@@ -220,74 +293,16 @@ typedef long mp_off_t;
 
 // extra built in modules to add to the list of known ones
 extern const struct _mp_obj_module_t board_module;
-extern const struct _mp_obj_module_t machine_module;
 extern const struct _mp_obj_module_t nrf_module;
 extern const struct _mp_obj_module_t mp_module_utime;
 extern const struct _mp_obj_module_t mp_module_uos;
 extern const struct _mp_obj_module_t mp_module_ubluepy;
 extern const struct _mp_obj_module_t music_module;
 
-#if MICROPY_PY_NRF
-#define NRF_MODULE                          { MP_ROM_QSTR(MP_QSTR_nrf), MP_ROM_PTR(&nrf_module) },
-#else
-#define NRF_MODULE
-#endif
-
-#if MICROPY_PY_UBLUEPY
-#define UBLUEPY_MODULE                      { MP_ROM_QSTR(MP_QSTR_ubluepy), MP_ROM_PTR(&mp_module_ubluepy) },
-#else
-#define UBLUEPY_MODULE
-#endif
-
-#if MICROPY_PY_MUSIC
-#define MUSIC_MODULE                        { MP_ROM_QSTR(MP_QSTR_music), MP_ROM_PTR(&music_module) },
-#else
-#define MUSIC_MODULE
-#endif
-
 #if BOARD_SPECIFIC_MODULES
 #include "boardmodules.h"
-#define MICROPY_BOARD_BUILTINS BOARD_MODULES
-#else
-#define MICROPY_BOARD_BUILTINS
 #endif // BOARD_SPECIFIC_MODULES
 
-#if BLUETOOTH_SD
-
-#if MICROPY_PY_BLE
-extern const struct _mp_obj_module_t ble_module;
-#define BLE_MODULE                        { MP_ROM_QSTR(MP_QSTR_ble), MP_ROM_PTR(&ble_module) },
-#else
-#define BLE_MODULE
-#endif
-
-#define MICROPY_PORT_BUILTIN_MODULES \
-    { MP_ROM_QSTR(MP_QSTR_board), MP_ROM_PTR(&board_module) }, \
-    { MP_ROM_QSTR(MP_QSTR_machine), MP_ROM_PTR(&machine_module) }, \
-    { MP_ROM_QSTR(MP_QSTR_utime), MP_ROM_PTR(&mp_module_utime) }, \
-    { MP_ROM_QSTR(MP_QSTR_time), MP_ROM_PTR(&mp_module_utime) }, \
-    { MP_ROM_QSTR(MP_QSTR_uos), MP_ROM_PTR(&mp_module_uos) }, \
-    BLE_MODULE \
-    MUSIC_MODULE \
-    UBLUEPY_MODULE \
-    MICROPY_BOARD_BUILTINS \
-    NRF_MODULE \
-
-
-#else
-extern const struct _mp_obj_module_t ble_module;
-#define MICROPY_PORT_BUILTIN_MODULES \
-    { MP_ROM_QSTR(MP_QSTR_board), MP_ROM_PTR(&board_module) }, \
-    { MP_ROM_QSTR(MP_QSTR_machine), MP_ROM_PTR(&machine_module) }, \
-    { MP_ROM_QSTR(MP_QSTR_utime), MP_ROM_PTR(&mp_module_utime) }, \
-    { MP_ROM_QSTR(MP_QSTR_uos), MP_ROM_PTR(&mp_module_uos) }, \
-    MUSIC_MODULE \
-    MICROPY_BOARD_BUILTINS \
-    NRF_MODULE \
-
-
-#endif // BLUETOOTH_SD
-
 // extra built in names to add to the global namespace
 #define MICROPY_PORT_BUILTINS \
     { MP_ROM_QSTR(MP_QSTR_help), MP_ROM_PTR(&mp_builtin_help_obj) }, \
@@ -296,8 +311,7 @@ extern const struct _mp_obj_module_t ble_module;
 // extra constants
 #define MICROPY_PORT_CONSTANTS \
     { MP_ROM_QSTR(MP_QSTR_board), MP_ROM_PTR(&board_module) }, \
-    { MP_ROM_QSTR(MP_QSTR_machine), MP_ROM_PTR(&machine_module) }, \
-    BLE_MODULE \
+    { MP_ROM_QSTR(MP_QSTR_machine), MP_ROM_PTR(&mp_module_machine) }, \
 
 #define MP_STATE_PORT MP_STATE_VM
 
