@@ -34,6 +34,7 @@
 #include "hardware/irq.h"
 #include "hardware/uart.h"
 #include "hardware/regs/uart.h"
+#include "pico/mutex.h"
 
 #define DEFAULT_UART_BAUDRATE (115200)
 #define DEFAULT_UART_BITS (8)
@@ -72,6 +73,16 @@
 #define UART_HWCONTROL_CTS  (1)
 #define UART_HWCONTROL_RTS  (2)
 
+STATIC mutex_t write_mutex_0;
+STATIC mutex_t write_mutex_1;
+STATIC mutex_t read_mutex_0;
+STATIC mutex_t read_mutex_1;
+
+auto_init_mutex(write_mutex_0);
+auto_init_mutex(write_mutex_1);
+auto_init_mutex(read_mutex_0);
+auto_init_mutex(read_mutex_1);
+
 typedef struct _machine_uart_obj_t {
     mp_obj_base_t base;
     uart_inst_t *const uart;
@@ -89,18 +100,18 @@ typedef struct _machine_uart_obj_t {
     uint8_t invert;
     uint8_t flow;
     ringbuf_t read_buffer;
-    bool read_lock;
+    mutex_t *read_mutex;
     ringbuf_t write_buffer;
-    bool write_lock;
+    mutex_t *write_mutex;
 } machine_uart_obj_t;
 
 STATIC machine_uart_obj_t machine_uart_obj[] = {
     {{&machine_uart_type}, uart0, 0, 0, DEFAULT_UART_BITS, UART_PARITY_NONE, DEFAULT_UART_STOP,
      MICROPY_HW_UART0_TX, MICROPY_HW_UART0_RX, MICROPY_HW_UART0_CTS, MICROPY_HW_UART0_RTS,
-     0, 0, 0, 0, {NULL, 1, 0, 0}, 0, {NULL, 1, 0, 0}, 0},
+     0, 0, 0, 0, {NULL, 1, 0, 0}, &read_mutex_0, {NULL, 1, 0, 0}, &write_mutex_0},
     {{&machine_uart_type}, uart1, 1, 0, DEFAULT_UART_BITS, UART_PARITY_NONE, DEFAULT_UART_STOP,
      MICROPY_HW_UART1_TX, MICROPY_HW_UART1_RX, MICROPY_HW_UART1_CTS, MICROPY_HW_UART1_RTS,
-     0, 0, 0, 0, {NULL, 1, 0, 0}, 0, {NULL, 1, 0, 0}, 0},
+     0, 0, 0, 0, {NULL, 1, 0, 0}, &read_mutex_1, {NULL, 1, 0, 0}, &write_mutex_1},
 };
 
 STATIC const char *_parity_name[] = {"None", "0", "1"};
@@ -109,36 +120,55 @@ STATIC const char *_invert_name[] = {"None", "INV_TX", "INV_RX", "INV_TX|INV_RX"
 /******************************************************************************/
 // IRQ and buffer handling
 
-// take all bytes from the fifo and store them, if possible, in the buffer
+static inline bool write_mutex_try_lock(machine_uart_obj_t *u) {
+    return mutex_enter_timeout_ms(u->write_mutex, 0);
+}
+
+static inline void write_mutex_unlock(machine_uart_obj_t *u) {
+    mutex_exit(u->write_mutex);
+}
+
+static inline bool read_mutex_try_lock(machine_uart_obj_t *u) {
+    return mutex_enter_timeout_ms(u->read_mutex, 0);
+}
+
+static inline void read_mutex_unlock(machine_uart_obj_t *u) {
+    mutex_exit(u->read_mutex);
+}
+
+// take all bytes from the fifo and store them in the buffer
 STATIC void uart_drain_rx_fifo(machine_uart_obj_t *self) {
-    while (uart_is_readable(self->uart)) {
-        // try to write the data, ignore the fail
-        ringbuf_put(&(self->read_buffer), uart_get_hw(self->uart)->dr);
+    if (read_mutex_try_lock(self)) {
+        while (uart_is_readable(self->uart) && ringbuf_free(&self->read_buffer) > 0) {
+            // get a byte from uart and put into the buffer
+            ringbuf_put(&(self->read_buffer), uart_get_hw(self->uart)->dr);
+        }
+        read_mutex_unlock(self);
     }
 }
 
 // take bytes from the buffer and put them into the UART FIFO
+// Re-entrancy: quit if an instance already running
 STATIC void uart_fill_tx_fifo(machine_uart_obj_t *self) {
-    while (uart_is_writable(self->uart) && ringbuf_avail(&self->write_buffer) > 0) {
-        // get a byte from the buffer and put it into the uart
-        uart_get_hw(self->uart)->dr = ringbuf_get(&(self->write_buffer));
+    if (write_mutex_try_lock(self)) {
+        while (uart_is_writable(self->uart) && ringbuf_avail(&self->write_buffer) > 0) {
+            // get a byte from the buffer and put it into the uart
+            uart_get_hw(self->uart)->dr = ringbuf_get(&(self->write_buffer));
+        }
+        write_mutex_unlock(self);
     }
 }
 
 STATIC inline void uart_service_interrupt(machine_uart_obj_t *self) {
-    if (uart_get_hw(self->uart)->mis & UART_UARTMIS_RXMIS_BITS) { // rx interrupt?
+    if (uart_get_hw(self->uart)->mis & (UART_UARTMIS_RXMIS_BITS | UART_UARTMIS_RTMIS_BITS)) { // rx interrupt?
         // clear all interrupt bits but tx
         uart_get_hw(self->uart)->icr = UART_UARTICR_BITS & (~UART_UARTICR_TXIC_BITS);
-        if (!self->read_lock) {
-            uart_drain_rx_fifo(self);
-        }
+        uart_drain_rx_fifo(self);
     }
     if (uart_get_hw(self->uart)->mis & UART_UARTMIS_TXMIS_BITS) { // tx interrupt?
         // clear all interrupt bits but rx
         uart_get_hw(self->uart)->icr = UART_UARTICR_BITS & (~UART_UARTICR_RXIC_BITS);
-        if (!self->write_lock) {
-            uart_fill_tx_fifo(self);
-        }
+        uart_fill_tx_fifo(self);
     }
 }
 
@@ -162,11 +192,10 @@ STATIC void machine_uart_print(const mp_print_t *print, mp_obj_t self_in, mp_pri
         self->timeout, self->timeout_char, _invert_name[self->invert]);
 }
 
-STATIC mp_obj_t machine_uart_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *all_args) {
-    enum { ARG_id, ARG_baudrate, ARG_bits, ARG_parity, ARG_stop, ARG_tx, ARG_rx, ARG_cts, ARG_rts,
+STATIC void machine_uart_init_helper(machine_uart_obj_t *self, size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+    enum { ARG_baudrate, ARG_bits, ARG_parity, ARG_stop, ARG_tx, ARG_rx, ARG_cts, ARG_rts,
            ARG_timeout, ARG_timeout_char, ARG_invert, ARG_flow, ARG_txbuf, ARG_rxbuf};
     static const mp_arg_t allowed_args[] = {
-        { MP_QSTR_id, MP_ARG_REQUIRED | MP_ARG_OBJ, {.u_rom_obj = MP_ROM_NONE} },
         { MP_QSTR_baudrate, MP_ARG_INT, {.u_int = -1} },
         { MP_QSTR_bits, MP_ARG_INT, {.u_int = -1} },
         { MP_QSTR_parity, MP_ARG_OBJ, {.u_rom_obj = MP_ROM_INT(-1)} },
@@ -185,16 +214,7 @@ STATIC mp_obj_t machine_uart_make_new(const mp_obj_type_t *type, size_t n_args,
 
     // Parse args.
     mp_arg_val_t args[MP_ARRAY_SIZE(allowed_args)];
-    mp_arg_parse_all_kw_array(n_args, n_kw, all_args, MP_ARRAY_SIZE(allowed_args), allowed_args, args);
-
-    // Get UART bus.
-    int uart_id = mp_obj_get_int(args[ARG_id].u_obj);
-    if (uart_id < 0 || uart_id >= MP_ARRAY_SIZE(machine_uart_obj)) {
-        mp_raise_msg_varg(&mp_type_ValueError, MP_ERROR_TEXT("UART(%d) doesn't exist"), uart_id);
-    }
-
-    // Get static peripheral object.
-    machine_uart_obj_t *self = (machine_uart_obj_t *)&machine_uart_obj[uart_id];
+    mp_arg_parse_all(n_args, pos_args, kw_args, MP_ARRAY_SIZE(allowed_args), allowed_args, args);
 
     // Set baudrate if configured.
     if (args[ARG_baudrate].u_int > 0) {
@@ -280,8 +300,6 @@ STATIC mp_obj_t machine_uart_make_new(const mp_obj_type_t *type, size_t n_args,
         self->flow = args[ARG_flow].u_int;
     }
 
-    self->read_lock = false;
-
     // Set the RX buffer size if configured.
     size_t rxbuf_len = DEFAULT_BUFFER_SIZE;
     if (args[ARG_rxbuf].u_int > 0) {
@@ -305,7 +323,7 @@ STATIC mp_obj_t machine_uart_make_new(const mp_obj_type_t *type, size_t n_args,
     }
 
     // Initialise the UART peripheral if any arguments given, or it was not initialised previously.
-    if (n_args > 1 || n_kw > 0 || self->baudrate == 0) {
+    if (n_args > 0 || kw_args->used > 0 || self->baudrate == 0) {
         if (self->baudrate == 0) {
             self->baudrate = DEFAULT_UART_BAUDRATE;
         }
@@ -339,10 +357,10 @@ STATIC mp_obj_t machine_uart_make_new(const mp_obj_type_t *type, size_t n_args,
 
         // Allocate the RX/TX buffers.
         ringbuf_alloc(&(self->read_buffer), rxbuf_len + 1);
-        MP_STATE_PORT(rp2_uart_rx_buffer[uart_id]) = self->read_buffer.buf;
+        MP_STATE_PORT(rp2_uart_rx_buffer[self->uart_id]) = self->read_buffer.buf;
 
         ringbuf_alloc(&(self->write_buffer), txbuf_len + 1);
-        MP_STATE_PORT(rp2_uart_tx_buffer[uart_id]) = self->write_buffer.buf;
+        MP_STATE_PORT(rp2_uart_tx_buffer[self->uart_id]) = self->write_buffer.buf;
 
         // Set the irq handler.
         if (self->uart_id == 0) {
@@ -356,16 +374,54 @@ STATIC mp_obj_t machine_uart_make_new(const mp_obj_type_t *type, size_t n_args,
         // Enable the uart irq; this macro sets the rx irq level to 4.
         uart_set_irq_enables(self->uart, true, true);
     }
+}
+
+STATIC mp_obj_t machine_uart_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *args) {
+    mp_arg_check_num(n_args, n_kw, 1, MP_OBJ_FUN_ARGS_MAX, true);
+
+    // Get UART bus.
+    int uart_id = mp_obj_get_int(args[0]);
+    if (uart_id < 0 || uart_id >= MP_ARRAY_SIZE(machine_uart_obj)) {
+        mp_raise_msg_varg(&mp_type_ValueError, MP_ERROR_TEXT("UART(%d) doesn't exist"), uart_id);
+    }
+
+    // Get static peripheral object.
+    machine_uart_obj_t *self = (machine_uart_obj_t *)&machine_uart_obj[uart_id];
+
+    // Initialise the UART peripheral.
+    mp_map_t kw_args;
+    mp_map_init_fixed_table(&kw_args, n_kw, args + n_args);
+    machine_uart_init_helper(self, n_args - 1, args + 1, &kw_args);
 
     return MP_OBJ_FROM_PTR(self);
 }
 
+STATIC mp_obj_t machine_uart_init(size_t n_args, const mp_obj_t *args, mp_map_t *kw_args) {
+    // Initialise the UART peripheral.
+    machine_uart_init_helper(args[0], n_args - 1, args + 1, kw_args);
+    return mp_const_none;
+}
+MP_DEFINE_CONST_FUN_OBJ_KW(machine_uart_init_obj, 1, machine_uart_init);
+
+STATIC mp_obj_t machine_uart_deinit(mp_obj_t self_in) {
+    machine_uart_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    uart_deinit(self->uart);
+    if (self->uart_id == 0) {
+        irq_set_enabled(UART0_IRQ, false);
+    } else {
+        irq_set_enabled(UART1_IRQ, false);
+    }
+    self->baudrate = 0;
+    MP_STATE_PORT(rp2_uart_rx_buffer[self->uart_id]) = NULL;
+    MP_STATE_PORT(rp2_uart_tx_buffer[self->uart_id]) = NULL;
+    return mp_const_none;
+}
+STATIC MP_DEFINE_CONST_FUN_OBJ_1(machine_uart_deinit_obj, machine_uart_deinit);
+
 STATIC mp_obj_t machine_uart_any(mp_obj_t self_in) {
     machine_uart_obj_t *self = MP_OBJ_TO_PTR(self_in);
     // get all bytes from the fifo first
-    self->read_lock = true;
     uart_drain_rx_fifo(self);
-    self->read_lock = false;
     return MP_OBJ_NEW_SMALL_INT(ringbuf_avail(&self->read_buffer));
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(machine_uart_any_obj, machine_uart_any);
@@ -380,8 +436,10 @@ STATIC mp_obj_t machine_uart_sendbreak(mp_obj_t self_in) {
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(machine_uart_sendbreak_obj, machine_uart_sendbreak);
 
 STATIC const mp_rom_map_elem_t machine_uart_locals_dict_table[] = {
-    { MP_ROM_QSTR(MP_QSTR_any), MP_ROM_PTR(&machine_uart_any_obj) },
+    { MP_ROM_QSTR(MP_QSTR_init), MP_ROM_PTR(&machine_uart_init_obj) },
+    { MP_ROM_QSTR(MP_QSTR_deinit), MP_ROM_PTR(&machine_uart_deinit_obj) },
 
+    { MP_ROM_QSTR(MP_QSTR_any), MP_ROM_PTR(&machine_uart_any_obj) },
     { MP_ROM_QSTR(MP_QSTR_read), MP_ROM_PTR(&mp_stream_read_obj) },
     { MP_ROM_QSTR(MP_QSTR_readline), MP_ROM_PTR(&mp_stream_unbuffered_readline_obj) },
     { MP_ROM_QSTR(MP_QSTR_readinto), MP_ROM_PTR(&mp_stream_readinto_obj) },
@@ -409,9 +467,7 @@ STATIC mp_uint_t machine_uart_read(mp_obj_t self_in, void *buf_in, mp_uint_t siz
         while (ringbuf_avail(&self->read_buffer) == 0) {
             if (uart_is_readable(self->uart)) {
                 // Force a few incoming bytes to the buffer
-                self->read_lock = true;
                 uart_drain_rx_fifo(self);
-                self->read_lock = false;
                 break;
             }
             if (time_us_64() > t) {  // timed out
@@ -444,9 +500,7 @@ STATIC mp_uint_t machine_uart_write(mp_obj_t self_in, const void *buf_in, mp_uin
     }
 
     // Kickstart the UART transmit.
-    self->write_lock = true;
     uart_fill_tx_fifo(self);
-    self->write_lock = false;
 
     // Send the next characters while busy waiting.
     while (i < size) {
@@ -465,9 +519,7 @@ STATIC mp_uint_t machine_uart_write(mp_obj_t self_in, const void *buf_in, mp_uin
         ringbuf_put(&(self->write_buffer), *src++);
         ++i;
         t = time_us_64() + timeout_char_us;
-        self->write_lock = true;
         uart_fill_tx_fifo(self);
-        self->write_lock = false;
     }
 
     // Just in case the fifo was drained during refill of the ringbuf.
