@@ -27,52 +27,76 @@
 #include "py/runtime.h"
 #include "py/stream.h"
 #include "py/mphal.h"
+#include "extmod/misc.h"
+#include "shared/runtime/interrupt_char.h"
 #include "shared/timeutils/timeutils.h"
 #include "tusb.h"
 #include "uart.h"
 #include "hardware/rtc.h"
+#include "pico/unique_id.h"
 
-#if MICROPY_HW_ENABLE_UART_REPL
+#if MICROPY_HW_ENABLE_UART_REPL || MICROPY_HW_ENABLE_USBDEV
 
-#ifndef UART_BUFFER_LEN
-// reasonably big so we can paste
-#define UART_BUFFER_LEN 256
+#ifndef MICROPY_HW_STDIN_BUFFER_LEN
+#define MICROPY_HW_STDIN_BUFFER_LEN 512
 #endif
 
-STATIC uint8_t stdin_ringbuf_array[UART_BUFFER_LEN];
+STATIC uint8_t stdin_ringbuf_array[MICROPY_HW_STDIN_BUFFER_LEN];
 ringbuf_t stdin_ringbuf = { stdin_ringbuf_array, sizeof(stdin_ringbuf_array) };
 
 #endif
 
-#if MICROPY_KBD_EXCEPTION
+#if MICROPY_HW_ENABLE_USBDEV
 
-int mp_interrupt_char = -1;
+uint8_t cdc_itf_pending; // keep track of cdc interfaces which need attention to poll
 
-void tud_cdc_rx_wanted_cb(uint8_t itf, char wanted_char) {
-    (void)itf;
-    (void)wanted_char;
-    tud_cdc_read_char(); // discard interrupt char
-    mp_sched_keyboard_interrupt();
+void poll_cdc_interfaces(void) {
+    // any CDC interfaces left to poll?
+    if (cdc_itf_pending && ringbuf_free(&stdin_ringbuf)) {
+        for (uint8_t itf = 0; itf < 8; ++itf) {
+            if (cdc_itf_pending & (1 << itf)) {
+                tud_cdc_rx_cb(itf);
+                if (!cdc_itf_pending) {
+                    break;
+                }
+            }
+        }
+    }
 }
 
-void mp_hal_set_interrupt_char(int c) {
-    mp_interrupt_char = c;
-    tud_cdc_set_wanted_char(c);
+void tud_cdc_rx_cb(uint8_t itf) {
+    // consume pending USB data immediately to free usb buffer and keep the endpoint from stalling.
+    // in case the ringbuffer is full, mark the CDC interface that need attention later on for polling
+    cdc_itf_pending &= ~(1 << itf);
+    for (uint32_t bytes_avail = tud_cdc_n_available(itf); bytes_avail > 0; --bytes_avail) {
+        if (ringbuf_free(&stdin_ringbuf)) {
+            int data_char = tud_cdc_read_char();
+            if (data_char == mp_interrupt_char) {
+                mp_sched_keyboard_interrupt();
+            } else {
+                ringbuf_put(&stdin_ringbuf, data_char);
+            }
+        } else {
+            cdc_itf_pending |= (1 << itf);
+            return;
+        }
+    }
 }
 
 #endif
 
 uintptr_t mp_hal_stdio_poll(uintptr_t poll_flags) {
     uintptr_t ret = 0;
-    #if MICROPY_HW_ENABLE_UART_REPL
+    #if MICROPY_HW_ENABLE_USBDEV
+    poll_cdc_interfaces();
+    #endif
+    #if MICROPY_HW_ENABLE_UART_REPL || MICROPY_HW_ENABLE_USBDEV
     if ((poll_flags & MP_STREAM_POLL_RD) && ringbuf_peek(&stdin_ringbuf) != -1) {
         ret |= MP_STREAM_POLL_RD;
     }
     #endif
-    #if MICROPY_HW_ENABLE_USBDEV
-    if (tud_cdc_connected() && tud_cdc_available()) {
-        ret |= MP_STREAM_POLL_RD;
-    }
+    #if MICROPY_PY_OS_DUPTERM
+    ret |= mp_uos_dupterm_poll(poll_flags);
     #endif
     return ret;
 }
@@ -80,19 +104,18 @@ uintptr_t mp_hal_stdio_poll(uintptr_t poll_flags) {
 // Receive single character
 int mp_hal_stdin_rx_chr(void) {
     for (;;) {
-        #if MICROPY_HW_ENABLE_UART_REPL
+        #if MICROPY_HW_ENABLE_USBDEV
+        poll_cdc_interfaces();
+        #endif
+
         int c = ringbuf_get(&stdin_ringbuf);
         if (c != -1) {
             return c;
         }
-        #endif
-        #if MICROPY_HW_ENABLE_USBDEV
-        if (tud_cdc_connected() && tud_cdc_available()) {
-            uint8_t buf[1];
-            uint32_t count = tud_cdc_read(buf, sizeof(buf));
-            if (count) {
-                return buf[0];
-            }
+        #if MICROPY_PY_OS_DUPTERM
+        int dupterm_c = mp_uos_dupterm_rx_chr();
+        if (dupterm_c >= 0) {
+            return dupterm_c;
         }
         #endif
         MICROPY_EVENT_POLL_HOOK
@@ -113,16 +136,18 @@ void mp_hal_stdout_tx_strn(const char *str, mp_uint_t len) {
                 n = CFG_TUD_CDC_EP_BUFSIZE;
             }
             while (n > tud_cdc_write_available()) {
-                tud_task();
-                tud_cdc_write_flush();
+                MICROPY_EVENT_POLL_HOOK
             }
             uint32_t n2 = tud_cdc_write(str + i, n);
-            tud_task();
             tud_cdc_write_flush();
             i += n2;
         }
     }
     #endif
+
+    #if MICROPY_PY_OS_DUPTERM
+    mp_uos_dupterm_tx_strn(str, len);
+    #endif
 }
 
 void mp_hal_delay_ms(mp_uint_t ms) {
@@ -140,3 +165,20 @@ uint64_t mp_hal_time_ns(void) {
     uint64_t s = timeutils_seconds_since_epoch(t.year, t.month, t.day, t.hour, t.min, t.sec);
     return s * 1000000000ULL;
 }
+
+// Generate a random locally administered MAC address (LAA)
+void mp_hal_generate_laa_mac(int idx, uint8_t buf[6]) {
+    pico_unique_board_id_t pid;
+    pico_get_unique_board_id(&pid);
+    buf[0] = 0x02; // LAA range
+    buf[1] = (pid.id[7] << 4) | (pid.id[6] & 0xf);
+    buf[2] = (pid.id[5] << 4) | (pid.id[4] & 0xf);
+    buf[3] = (pid.id[3] << 4) | (pid.id[2] & 0xf);
+    buf[4] = pid.id[1];
+    buf[5] = (pid.id[0] << 2) | idx;
+}
+
+// A board can override this if needed
+MP_WEAK void mp_hal_get_mac(int idx, uint8_t buf[6]) {
+    mp_hal_generate_laa_mac(idx, buf);
+}
