@@ -29,10 +29,10 @@
 
 #include "py/runtime.h"
 #include "py/mphal.h"
+#include "extmod/modnetwork.h"
 #include "spi.h"
-#include "modnetwork.h"
 
-#if MICROPY_PY_WIZNET5K && MICROPY_PY_LWIP
+#if MICROPY_PY_NETWORK_WIZNET5K && MICROPY_PY_LWIP
 
 #include "shared/netutils/netutils.h"
 #include "drivers/wiznet5k/ethernet/socket.h"
@@ -269,7 +269,7 @@ STATIC mp_obj_t wiznet5k_make_new(const mp_obj_type_t *type, size_t n_args, size
     mp_hal_pin_obj_t rst = pin_find(args[2]);
 
     // Access the existing object, if it has been constructed with the same hardware interface
-    if (wiznet5k_obj.base.type == &mod_network_nic_type_wiznet5k) {
+    if (wiznet5k_obj.base.type == (mp_obj_type_t *)&mod_network_nic_type_wiznet5k) {
         if (!(wiznet5k_obj.spi == spi && wiznet5k_obj.cs == cs && wiznet5k_obj.rst == rst
               && wiznet5k_obj.netif.flags != 0)) {
             wiznet5k_deinit();
@@ -277,7 +277,7 @@ STATIC mp_obj_t wiznet5k_make_new(const mp_obj_type_t *type, size_t n_args, size
     }
 
     // Init the wiznet5k object
-    wiznet5k_obj.base.type = &mod_network_nic_type_wiznet5k;
+    wiznet5k_obj.base.type = (mp_obj_type_t *)&mod_network_nic_type_wiznet5k;
     wiznet5k_obj.cris_state = 0;
     wiznet5k_obj.spi = spi;
     wiznet5k_obj.cs = cs;
@@ -295,7 +295,7 @@ STATIC mp_obj_t wiznet5k_regs(mp_obj_t self_in) {
         if (i % 16 == 0) {
             printf("\n  %04x:", i);
         }
-        #if MICROPY_PY_WIZNET5K == 5200
+        #if MICROPY_PY_NETWORK_WIZNET5K == 5200
         uint32_t reg = i;
         #else
         uint32_t reg = _W5500_IO_BASE_ | i << 8;
@@ -308,7 +308,7 @@ STATIC mp_obj_t wiznet5k_regs(mp_obj_t self_in) {
             if (i % 16 == 0) {
                 printf("\n  %04x:", i);
             }
-            #if MICROPY_PY_WIZNET5K == 5200
+            #if MICROPY_PY_NETWORK_WIZNET5K == 5200
             uint32_t reg = WIZCHIP_SREG_ADDR(sn, i);
             #else
             uint32_t reg = _W5500_IO_BASE_ | i << 8 | WIZCHIP_SREG_BLOCK(sn) << 3;
@@ -462,4 +462,4 @@ const mp_obj_type_t mod_network_nic_type_wiznet5k = {
     .locals_dict = (mp_obj_dict_t *)&wiznet5k_locals_dict,
 };
 
-#endif // MICROPY_PY_WIZNET5K && MICROPY_PY_LWIP
+#endif // MICROPY_PY_NETWORK_WIZNET5K && MICROPY_PY_LWIP
