@@ -60,6 +60,10 @@
 #define STATIC_AF_SDCARD_D1 STATIC_AF_SDMMC2_D1
 #define STATIC_AF_SDCARD_D2 STATIC_AF_SDMMC2_D2
 #define STATIC_AF_SDCARD_D3 STATIC_AF_SDMMC2_D3
+#define STATIC_AF_SDCARD_D4 STATIC_AF_SDMMC2_D4
+#define STATIC_AF_SDCARD_D5 STATIC_AF_SDMMC2_D5
+#define STATIC_AF_SDCARD_D6 STATIC_AF_SDMMC2_D6
+#define STATIC_AF_SDCARD_D7 STATIC_AF_SDMMC2_D7
 #else
 #define SDIO SDMMC1
 #define SDMMC_IRQHandler SDMMC1_IRQHandler
@@ -75,6 +79,10 @@
 #define STATIC_AF_SDCARD_D1 STATIC_AF_SDMMC1_D1
 #define STATIC_AF_SDCARD_D2 STATIC_AF_SDMMC1_D2
 #define STATIC_AF_SDCARD_D3 STATIC_AF_SDMMC1_D3
+#define STATIC_AF_SDCARD_D4 STATIC_AF_SDMMC1_D4
+#define STATIC_AF_SDCARD_D5 STATIC_AF_SDMMC1_D5
+#define STATIC_AF_SDCARD_D6 STATIC_AF_SDMMC1_D6
+#define STATIC_AF_SDCARD_D7 STATIC_AF_SDMMC1_D7
 #endif
 
 // The F7 & L4 series calls the peripheral SDMMC rather than SDIO, so provide some
@@ -120,6 +128,10 @@
 #define STATIC_AF_SDCARD_D1 STATIC_AF_SDIO_D1
 #define STATIC_AF_SDCARD_D2 STATIC_AF_SDIO_D2
 #define STATIC_AF_SDCARD_D3 STATIC_AF_SDIO_D3
+#define STATIC_AF_SDCARD_D4 STATIC_AF_SDIO_D4
+#define STATIC_AF_SDCARD_D5 STATIC_AF_SDIO_D5
+#define STATIC_AF_SDCARD_D6 STATIC_AF_SDIO_D6
+#define STATIC_AF_SDCARD_D7 STATIC_AF_SDIO_D7
 
 #endif
 
@@ -133,6 +145,13 @@
 #define MICROPY_HW_SDCARD_CMD (pin_D2)
 #endif
 
+// Define a constant to select the bus width.
+#if MICROPY_HW_SDCARD_BUS_WIDTH == 4
+#define SDIO_BUS_WIDE_VALUE SDIO_BUS_WIDE_4B
+#elif MICROPY_HW_SDCARD_BUS_WIDTH == 8
+#define SDIO_BUS_WIDE_VALUE SDIO_BUS_WIDE_8B
+#endif
+
 #define PYB_SDMMC_FLAG_SD       (0x01)
 #define PYB_SDMMC_FLAG_MMC      (0x02)
 #define PYB_SDMMC_FLAG_ACTIVE   (0x04)
@@ -162,10 +181,16 @@ void sdcard_init(void) {
     mp_hal_pin_config_alt_static(MICROPY_HW_SDCARD_CK, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_UP, STATIC_AF_SDCARD_CK);
     mp_hal_pin_config_alt_static(MICROPY_HW_SDCARD_CMD, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_UP, STATIC_AF_SDCARD_CMD);
     mp_hal_pin_config_alt_static(MICROPY_HW_SDCARD_D0, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_UP, STATIC_AF_SDCARD_D0);
-    #if MICROPY_HW_SDCARD_BUS_WIDTH == 4
+    #if MICROPY_HW_SDCARD_BUS_WIDTH >= 4
     mp_hal_pin_config_alt_static(MICROPY_HW_SDCARD_D1, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_UP, STATIC_AF_SDCARD_D1);
     mp_hal_pin_config_alt_static(MICROPY_HW_SDCARD_D2, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_UP, STATIC_AF_SDCARD_D2);
     mp_hal_pin_config_alt_static(MICROPY_HW_SDCARD_D3, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_UP, STATIC_AF_SDCARD_D3);
+    #if MICROPY_HW_SDCARD_BUS_WIDTH == 8
+    mp_hal_pin_config_alt_static(MICROPY_HW_SDCARD_D4, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_UP, STATIC_AF_SDCARD_D4);
+    mp_hal_pin_config_alt_static(MICROPY_HW_SDCARD_D5, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_UP, STATIC_AF_SDCARD_D5);
+    mp_hal_pin_config_alt_static(MICROPY_HW_SDCARD_D6, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_UP, STATIC_AF_SDCARD_D6);
+    mp_hal_pin_config_alt_static(MICROPY_HW_SDCARD_D7, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_UP, STATIC_AF_SDCARD_D7);
+    #endif
     #endif
 
     // configure the SD card detect pin
@@ -175,6 +200,14 @@ void sdcard_init(void) {
     #endif
 }
 
+void sdcard_select_sd(void) {
+    pyb_sdmmc_flags |= PYB_SDMMC_FLAG_SD;
+}
+
+void sdcard_select_mmc(void) {
+    pyb_sdmmc_flags |= PYB_SDMMC_FLAG_MMC;
+}
+
 STATIC void sdmmc_msp_init(void) {
     // enable SDIO clock
     SDMMC_CLK_ENABLE();
@@ -224,7 +257,7 @@ bool sdcard_is_present(void) {
     }
     #endif
     #if defined(MICROPY_HW_SDCARD_DETECT_PIN)
-    return HAL_GPIO_ReadPin(MICROPY_HW_SDCARD_DETECT_PIN->gpio, MICROPY_HW_SDCARD_DETECT_PIN->pin_mask) == MICROPY_HW_SDCARD_DETECT_PRESENT;
+    return mp_hal_pin_read(MICROPY_HW_SDCARD_DETECT_PIN) == MICROPY_HW_SDCARD_DETECT_PRESENT;
     #else
     return true;
     #endif
@@ -252,9 +285,9 @@ STATIC HAL_StatusTypeDef sdmmc_init_sd(void) {
         mp_hal_delay_ms(50);
     }
 
-    #if MICROPY_HW_SDCARD_BUS_WIDTH == 4
-    // configure the SD bus width for 4-bit wide operation
-    status = HAL_SD_ConfigWideBusOperation(&sdmmc_handle.sd, SDIO_BUS_WIDE_4B);
+    #if MICROPY_HW_SDCARD_BUS_WIDTH >= 4
+    // configure the SD bus width for 4/8-bit wide operation
+    status = HAL_SD_ConfigWideBusOperation(&sdmmc_handle.sd, SDIO_BUS_WIDE_VALUE);
     if (status != HAL_OK) {
         HAL_SD_DeInit(&sdmmc_handle.sd);
         return status;
@@ -284,15 +317,19 @@ STATIC HAL_StatusTypeDef sdmmc_init_mmc(void) {
         return status;
     }
 
-    // As this is an eMMC card, overwrite LogBlockNbr with actual value
-    sdmmc_handle.mmc.MmcCard.LogBlockNbr = 7469056 + 2048;
+    #ifdef MICROPY_HW_MMCARD_LOG_BLOCK_NBR
+    // A board can override the number of logical blocks (card capacity) if needed.
+    // This is needed when a card is high capacity because the extended CSD command
+    // is not supported by the current version of the HAL.
+    sdmmc_handle.mmc.MmcCard.LogBlockNbr = MICROPY_HW_MMCARD_LOG_BLOCK_NBR;
+    #endif
 
-    #if MICROPY_HW_SDCARD_BUS_WIDTH == 4
-    // Configure the SDIO bus width for 4-bit wide operation
+    #if MICROPY_HW_SDCARD_BUS_WIDTH >= 4
+    // Configure the SDIO bus width for 4/8-bit wide operation
     #ifdef STM32F7
     sdmmc_handle.mmc.Init.ClockBypass = SDIO_CLOCK_BYPASS_ENABLE;
     #endif
-    status = HAL_MMC_ConfigWideBusOperation(&sdmmc_handle.mmc, SDIO_BUS_WIDE_4B);
+    status = HAL_MMC_ConfigWideBusOperation(&sdmmc_handle.mmc, SDIO_BUS_WIDE_VALUE);
     if (status != HAL_OK) {
         HAL_MMC_DeInit(&sdmmc_handle.mmc);
         return status;
@@ -651,6 +688,8 @@ mp_uint_t sdcard_write_blocks(const uint8_t *src, uint32_t block_num, uint32_t n
 //
 // Expose the SD card or MMC as an object with the block protocol.
 
+#if !BUILDING_MBOOT
+
 // There are singleton SDCard/MMCard objects
 #if MICROPY_HW_ENABLE_SDCARD
 const mp_obj_base_t pyb_sdcard_obj = {&pyb_sdcard_type};
@@ -868,4 +907,6 @@ void sdcard_init_vfs(fs_user_mount_t *vfs, int part) {
     vfs->blockdev.u.ioctl[1] = MP_OBJ_FROM_PTR(&pyb_sdcard_obj);
 }
 
+#endif // !BUILDING_MBOOT
+
 #endif // MICROPY_HW_ENABLE_SDCARD || MICROPY_HW_ENABLE_MMCARD
