@@ -35,6 +35,8 @@
 
 #include <jni.h>
 
+#if MICROPY_PY_JNI
+
 #define JJ(call, ...) (*env)->call(env, __VA_ARGS__)
 #define JJ1(call) (*env)->call(env)
 #define MATCH(s, static) (!strncmp(s, static, sizeof(static) - 1))
@@ -145,8 +147,7 @@ STATIC void jclass_attr(mp_obj_t self_in, qstr attr_in, mp_obj_t *dest) {
         // JJ1(ExceptionDescribe);
         JJ1(ExceptionClear);
 
-        mp_obj_jmethod_t *o = m_new_obj(mp_obj_jmethod_t);
-        o->base.type = &jmethod_type;
+        mp_obj_jmethod_t *o = mp_obj_malloc(mp_obj_jmethod_t, &jmethod_type);
         o->name = attr_in;
         o->meth = NULL;
         o->obj = self->cls;
@@ -183,8 +184,7 @@ STATIC const mp_obj_type_t jclass_type = {
 };
 
 STATIC mp_obj_t new_jclass(jclass jc) {
-    mp_obj_jclass_t *o = m_new_obj(mp_obj_jclass_t);
-    o->base.type = &jclass_type;
+    mp_obj_jclass_t *o = mp_obj_malloc(mp_obj_jclass_t, &jclass_type);
     o->cls = jc;
     return MP_OBJ_FROM_PTR(o);
 }
@@ -223,8 +223,7 @@ STATIC void jobject_attr(mp_obj_t self_in, qstr attr_in, mp_obj_t *dest) {
         // JJ1(ExceptionDescribe);
         JJ1(ExceptionClear);
 
-        mp_obj_jmethod_t *o = m_new_obj(mp_obj_jmethod_t);
-        o->base.type = &jmethod_type;
+        mp_obj_jmethod_t *o = mp_obj_malloc(mp_obj_jmethod_t, &jmethod_type);
         o->name = attr_in;
         o->meth = NULL;
         o->obj = self->obj;
@@ -343,8 +342,7 @@ STATIC mp_obj_t new_jobject(jobject jo) {
     } else if (JJ(IsInstanceOf, jo, Class_class)) {
         return new_jclass(jo);
     } else {
-        mp_obj_jobject_t *o = m_new_obj(mp_obj_jobject_t);
-        o->base.type = &jobject_type;
+        mp_obj_jobject_t *o = mp_obj_malloc(mp_obj_jobject_t, &jobject_type);
         o->obj = jo;
         return MP_OBJ_FROM_PTR(o);
     }
@@ -644,8 +642,7 @@ STATIC mp_obj_t mod_jni_cls(mp_obj_t cls_name_in) {
     }
     jclass cls = JJ(FindClass, cls_name);
 
-    mp_obj_jclass_t *o = m_new_obj(mp_obj_jclass_t);
-    o->base.type = &jclass_type;
+    mp_obj_jclass_t *o = mp_obj_malloc(mp_obj_jclass_t, &jclass_type);
     o->cls = cls;
     return MP_OBJ_FROM_PTR(o);
 }
@@ -717,3 +714,7 @@ const mp_obj_module_t mp_module_jni = {
     .base = { &mp_type_module },
     .globals = (mp_obj_dict_t *)&mp_module_jni_globals,
 };
+
+MP_REGISTER_MODULE(MP_QSTR_jni, mp_module_jni);
+
+#endif // MICROPY_PY_JNI
